
/*
    IPX routing daemon

    Copyright (C) 1994, 1995  Ales Dryak <e-mail: A.Dryak@sh.cvut.cz>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/
/* done: filter my broadcasts for proper aging */
/* done: age each iface separately */
/* done: DONTROUTE ioctl */
/* done: shutdown */
/* done: logovani ... */
/* done: full daemonize */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <linux/ipx.h>
#include "ipxutil.h"
#include "ipxkern.h"
#include "ipxrip.h"

#if 1
#define TIMER_RATE	(30)
#define EXPIRE_TIME	(180)
#define BROADCAST_TIME	(60)
#define MAX_IFACE	(32)
#else
#define TIMER_RATE	(10)
#define EXPIRE_TIME	(180)
#define BROADCAST_TIME	(20)
#define MAX_IFACE	(32)
#endif

typedef unsigned char ifc_timer;

struct rip_table
{
	IPXNet network;
	hop_t hops;
	tick_t ticks;
	IPXNet router_network;
	IPXNode router_node;
	ifc_timer timers[MAX_IFACE]; /* 0<timer<EXPIRE_TIME =>
					net is reachable via iface */
				     /* timer>=EXPIPE_TIME => net is not r. */
	struct rip_table* next;
};

struct rip_interface
{
	IPXNode ifnode;
	struct rip_output output;
};

char in_buf[IPX_RIP_MAX_LENGTH];
struct rip_table  *rtable;
int n_of_ifaces;
struct rip_interface ifaces[MAX_IFACE];
int time_since_last_bcast=0;

volatile int dump_request=0;
volatile int timer_request=0;
volatile int terminate=0;

int new_log_entry=1;
int debug_option=0;
FILE *log_file=stderr;

#define LOG_ENTRY	{new_log_entry=1;}
#define LOG_START	{if (new_log_entry) {print_time(); new_log_entry=0;}
#define LOG_END		fflush(log_file);}
#define DL_ENTRY	{if (debug_option) LOG_ENTRY}
#define DL_START	{if (debug_option) LOG_START
#define DL_END		LOG_END }
#define DEFAULT_LOGNAME "/dev/null"

void
print_time()
{
	time_t t;
	time(&t);
	fprintf(log_file,"\n%s",ctime(&t));
}

static IPXNet
ifc_net(struct rip_interface *ifc)
{
	return ntohl(ifc->output.dest_addr.sipx_network);
}

static struct rip_interface*
get_iface(IPXNet net)
{
	struct rip_interface* ifc;
	for(ifc=ifaces; ifc<ifaces+n_of_ifaces; ifc += 1)
	{
		if (ifc_net(ifc)==net)
		{
			return ifc;
		}
	}
	return NULL;
}

static int
ifc_get_index(struct rip_interface *ifc)
{
	if (ifc==NULL)
	{
		return -1;
	}
	return ifc-ifaces;
}

/* get some ipx socket */
static int
ipx_socket(void)
{
	if (n_of_ifaces == 0)
	{
		LOG_START;
		fprintf(log_file, "No ipx socket available\n");
		LOG_END;
		exit(1);
	}
	return ifaces[0].output.sk;
}

static int
is_expired(ifc_timer* tm)
{
	return *tm>=EXPIRE_TIME;
}

/* Do I forward packets targeted to rt->network coming over interface
 * ifc?
 */
static int
is_rt_target(struct rip_table *rt, struct rip_interface *ifc)
{
	return is_expired(rt->timers + ifc_get_index(ifc));
}

/* set up the timers in rt so that only ifc is reachable.
 */
static void
setup_timers(struct rip_table* rt, struct rip_interface* ifc)
{
	ifc_timer* tm;
	
	for(tm=rt->timers; tm < rt->timers+MAX_IFACE; tm += 1)
	{
		*tm=EXPIRE_TIME;
	}
	rt->timers[ifc_get_index(ifc)]=0;
}

static struct rip_output *
get_output(int index)
{
	return &(ifaces[index].output);
}

static void
output_flushall()
{
	int i;
	for(i = 0; i<n_of_ifaces; i += 1)
	{
		ipx_rip_output_flush(get_output(i));
	}
}

static void
output_set_destination(IPXNode dest_node,IPXPort dest_port)
{
	int i;
	for(i=0; i<n_of_ifaces; i += 1)
	{
		ipx_rip_output_set_destination(get_output(i),dest_node,
					       dest_port);
	}
}

/* Broadcast the existence of the route rt to all interfaces through
 * which rt's target is not directly reachable.
 */
static void
output_broadcast(struct rip_table *rt, int down_allow)
{
	struct rip_interface *ifc;
	ifc_timer* tm;

	for(ifc=ifaces,tm=rt->timers;ifc<ifaces+n_of_ifaces;ifc++,tm++)
	{
		if (is_expired(tm))
		{
			ipx_rip_output_response(&(ifc->output),rt->network,
						rt->hops,rt->ticks,down_allow);
		}
	}
}

static void
output_sendto(int sock, void *buffer, int size, struct sockaddr_ipx *daddr)
{
	int res;
	
	DL_ENTRY;
	DL_START;
	fprintf(log_file,"Sending RIP to ");
	ipx_fprint_saddr(log_file,daddr);
	fprintf(log_file,"\n");
	ipx_rip_fdump(log_file,buffer,size);
	DL_END;

	res=sendto(sock,(void*)buffer,size,0,
		   (struct sockaddr*)daddr,sizeof(*daddr));
	if (res==-1)
	{
		LOG_START
		fprintf(log_file,"sendto: %s\n",strerror(errno));
		LOG_END
	}
}

static void
fprint_route(FILE *file, struct rip_table *rt)
{
	ifc_timer* tm;
	
	LOG_START;
	fprintf(file,"dst:");
	ipx_fprint_network(file,rt->network);
	fprintf(file," h:%d t:%d rtr:",rt->hops,rt->ticks);
	ipx_fprint_network(file,rt->router_network);
	fprintf(file,":");
	ipx_fprint_node(file,rt->router_node);
	fprintf(file," ");
	for(tm=rt->timers;tm<rt->timers+n_of_ifaces;tm++)
	{
		fprintf(file,"%i",is_expired(tm)?0:1);
		DL_START;
		fprintf(file,"(%i)",(int)*tm);
		DL_END;
	}
	LOG_END;
}

static void
fdump_routes(FILE *file)
{
	struct rip_table *rt;
	
	LOG_START;
	fprintf(file,"IPX routing database:\n");
	for(rt=rtable; rt!=NULL; rt=rt->next)
	{
		fprint_route(file,rt);
		fprintf(file,"\n");
	}
	LOG_END;
}


static struct rip_table*
add_route(IPXNet to, hop_t h, tick_t t, IPXNet rt_net, IPXNode rt_node)
{
	struct rip_table *rt;
	struct rip_interface *ifc;
	
	if ((ifc=get_iface(rt_net))==NULL)
	{
		return NULL;
	}

	rt=(struct rip_table*)malloc(sizeof(struct rip_table));

	if (rt == NULL)
	{
		LOG_START;
		fprintf(log_file,"ipxripd: out of memory in add_route\n");
		LOG_END;
		return NULL;
	}

	rt->network=to;
	rt->hops=h;
	rt->ticks=t;
	rt->router_network=rt_net;
	ipx_assign_node(rt->router_node,rt_node);
	setup_timers(rt,ifc);
	rt->next=rtable;
	rtable=rt;
	return rt;
}

static void
delete_route(struct rip_table* d)
{
	struct rip_table **r;
	for(r=&rtable; *r!=NULL; r=&((*r)->next))
	{
		if (*r==d)
		{
			*r=d->next;
			free(d);
			return;
		}
	}
}

static void
ipx_rip_rt_add(struct rip_table *rt)
{
	if(ipx_kern_route_add(ipx_socket(), rt->network, rt->router_network,
			      rt->router_node) < 0)
	{
		LOG_START;
		fprintf(log_file,"Cannot add route to network ");
		ipx_fprint_network(log_file,rt->network);
		fprintf(log_file,"\n%s\n",ipx_err_string);
		LOG_END;
	}
}

static void
ipx_rip_rt_del(IPXNet net)
{
	if (ipx_kern_route_delete(ipx_socket(), net)<0)
	{
		LOG_START;
		fprintf(log_file,"Cannot delete route to network ");
		ipx_fprint_network(log_file,net);
		fprintf(log_file,"\n%s\n",ipx_err_string);
		LOG_END;
	}
}

static void
handle_rip_request(struct rip_entry *re, struct rip_interface *src_ifc)
{
	struct rip_table *cur;

	if (re->network==htonl(IPX_RIP_GENERAL_RQ))
	{
		for(cur=rtable; cur!=NULL; cur=cur->next)
		{
			if (is_rt_target(cur, src_ifc))
			{
				ipx_rip_output_response(&(src_ifc->output),
							cur->network,
							cur->hops,
							cur->ticks,0);
			}
		}
	}
	else
	{
		for(cur=rtable; cur!=NULL; cur=cur->next)
		{
			if (   (htonl(re->network)==cur->network)
			    && (   (is_rt_target(cur, src_ifc))
				|| (   (ifc_net(src_ifc)==htonl(re->network))
				    && (get_iface(htonl(re->network))!=NULL))))
			{
				ipx_rip_output_response(&(src_ifc->output),
							cur->network,
							cur->hops,
							cur->ticks,0);
			}
		}
	}
}

static void
handle_rip_response(struct rip_entry *re, struct rip_interface *src_ifc,
		    IPXNode src_node)
{
	struct rip_table *cur;

	re->hops=ntohs(re->hops)+1;
	re->ticks=ntohs(re->ticks)+1;

	for(cur=rtable; cur!=NULL; cur=cur->next)
	{
		if (cur->network != htonl(re->network))
		{
			continue;
		}
		if (cur->network==cur->router_network)
		{
			/* don't change routes to ifaces */
			return;
		}
		if (re->hops <= IPX_RIP_NET_DOWN)
		{
			if (!(   (re->ticks < cur->ticks)
			      || (   (re->ticks == cur->ticks)
				  && (re->hops  <= cur->hops))))
			{
				/* route re-> is worse */
				return;
			}

			/* route is at least good as
			 * existing one */

			if (   (re->ticks==cur->ticks)
			    && (re->hops==cur->hops))
			{
				/* new route has the same cost - reset timer*/
				if (!(  (ifc_net(src_ifc)==cur->router_network)
				      && (!ipx_node_equal(cur->router_node,
							  src_node))))
				{
					cur->timers[ifc_get_index(src_ifc)]=0;
				}
			}
			else
			{
				/* new route is better */
				LOG_START;
				fprintf(log_file,"CHANGE ");
				fprint_route(log_file,cur);
				LOG_END;
						
				/* update table */
				cur->ticks=re->ticks;
				cur->hops=re->hops;
				cur->router_network=ifc_net(src_ifc);
				ipx_assign_node(cur->router_node,
						src_node);
				setup_timers(cur,src_ifc);

				LOG_START;
				fprintf(log_file,"\nto     ");
				fprint_route(log_file,cur);
				fprintf(log_file,"\n");
				LOG_END;
	
				/* update kernel table */
				ipx_rip_rt_add(cur);
				/* send info bcast */
				output_broadcast(cur,0);
			}
		}
		else
		{
			/* network down response */
			if (ifc_net(src_ifc)==cur->router_network)
			{
				if (ipx_node_equal(cur->router_node, src_node))
				{
					LOG_START;
					fprintf(log_file,"DELETE ");
					fprint_route(log_file,cur);
					fprintf(log_file, " (net down)\n");
					LOG_END;
					/* update kernel table */
					ipx_rip_rt_del(cur->network);
					/* send info bcast */
					cur->hops=IPX_RIP_NET_DOWN;
					output_broadcast(cur,1);
					/* update table */
					delete_route(cur);
				}
				else
				{ /* my router alive */
				}
			}
			else
			{
				cur->timers[ifc_get_index(src_ifc)]
					=EXPIRE_TIME;
			}
		}
		return;
	}
	if (re->hops<=IPX_RIP_NET_DOWN)
	{
		struct rip_table *rt=add_route(htonl(re->network),
					       re->hops,re->ticks,
					       ifc_net(src_ifc),src_node);
		if (rt!=NULL)
		{
			/* update kernel table */
			ipx_rip_rt_add(rt);
			/* send info bcast */
			output_broadcast(rt,0);
			LOG_START;
			fprintf(log_file,"ADD ");	
			fprint_route(log_file,rt);
			fprintf(log_file,"\n");
			LOG_END;
		}
	}
}

static void
handle_rip(struct rip_packet *pkt, int len, struct sockaddr_ipx* sipx,
	   struct rip_interface *src_ifc)
{
	struct rip_entry *re=pkt->rip_entries;
	int nent=(len-2)/8;

	if (ifc_net(src_ifc) != ntohl(sipx->sipx_network))
	{
		LOG_START;
		fprintf(log_file,"RIP from non-local net on ifc ");
		ipx_fprint_network(log_file, ifc_net(src_ifc));
		fprintf(log_file, "\n");
                ipx_fprint_network(log_file,ntohl(sipx->sipx_network));
		fprintf(log_file," (ignored)\n");
		LOG_END;
		return;
	}
	if (len<ipx_rip_size(1))
	{
		LOG_START;
		fprintf(log_file,
			"RIP packet too small len=%i (ignored)\n",len);
		LOG_END;
		return;
	}
	if (ipx_rip_size(nent)!=len)
	{
		LOG_START;
		fprintf(log_file,"RIP packet bad size (ignored)\n");
		LOG_END;
		return;
	}
	if (   (ipx_node_equal(src_ifc->ifnode,sipx->sipx_node))
	    && ((unsigned short)sipx->sipx_port==htons(IPX_RIP_PORT)))
	{
		DL_START;
		fprintf(log_file,"My packet (ignored)\n");
		DL_END;
		return;
	}
	switch (ntohs(pkt->operation))
	{
	case IPX_RIP_OP_REQUEST:
		output_set_destination(sipx->sipx_node,ntohs(sipx->sipx_port));
		for(;nent--;re++)
		{
			handle_rip_request(re,src_ifc);
		}
		output_flushall();
		break;
	case IPX_RIP_OP_RESPONSE:
		/* option: ignore responses from non RIP ports
		 * if (sipx->sipx_port!=htons(IPX_RIP_PORT)) return;
		 */
		output_set_destination(IPX_BROADCAST,IPX_RIP_PORT);
		for(;nent--;re++)
		{
			handle_rip_response(re,src_ifc,sipx->sipx_node);
		}
		output_flushall();
		break;
	default:
		LOG_START;
		fprintf(log_file,"Unknown RIP operation\n");
		LOG_END;
		break;
	} 
}

static void
do_aging()
{
	struct rip_table *cur;
	int do_broadcast = 0;
	int routes_died = 0;
	
	DL_START;
	fprintf(log_file,"DO AGING\n");
	DL_END;

	time_since_last_bcast += TIMER_RATE;
	if (time_since_last_bcast >= BROADCAST_TIME)
	{
		do_broadcast = 1;
		time_since_last_bcast -= BROADCAST_TIME;
	}
	output_set_destination(IPX_BROADCAST, IPX_RIP_PORT);

	for(cur=rtable; cur!=NULL; cur=cur->next)
	{
		ifc_timer* tm;
		
		for(tm=cur->timers; tm<cur->timers+n_of_ifaces; tm++)
		{
			if (!is_expired(tm))
			{
				(*tm)+=TIMER_RATE;
			}
		}
		tm=cur->timers+ifc_get_index(get_iface(cur->router_network));

		/* cannot timeout iface !*/
		if (cur->network==cur->router_network)
		{
			*tm=0;
		}

		if (is_expired(tm))
		{
			/* net is down */
			LOG_START;
			fprintf(log_file,"DELETE ");
			fprint_route(log_file,cur);
			fprintf(log_file," (timed out)\n");
			LOG_END;
			/* update kernel table */
			ipx_rip_rt_del(cur->network);
			/* send info bcast */
			cur->hops=IPX_RIP_NET_DOWN;
			output_broadcast(cur,1);
			/* table to be updated, delayed */
			routes_died = 1;
		}
		else
		{
			if (do_broadcast)
			{
				output_broadcast(cur,0);
			}
		}
	}
	output_flushall();

	/* Delete all bogus routes */
	if (routes_died != 0)
	{
		struct rip_table **r = &rtable;

		while (*r != NULL)
		{
			if ((*r)->hops >= IPX_RIP_NET_DOWN)
			{
				struct rip_table *d = *r;
				*r = (*r)->next;
				free(d);
			}
			else
			{
				r = &((*r)->next);
			}
		}
	}
}

static int
ipx_rip_create_ifc(IPXNet network, IPXNode node,
		   char *device, int type, void *data)
{
	struct rip_interface *ifc = &(ifaces[n_of_ifaces]);
	struct sockaddr_ipx sipx;

	if (n_of_ifaces >= MAX_IFACE)
	{
		LOG_START;
		fprintf(log_file,"too many interfaces (max. %i)\n",
			(int)MAX_IFACE);
		LOG_END;
		exit(1);
	}

	ipx_assign_node(ifc->ifnode, node);
	if (ipx_rip_output_rip_output(&(ifc->output), network) != 0)
	{
		LOG_START;
		fprintf(log_file,
			"out of memory allocating output buffer\n");
		LOG_END;
		exit(1);
	}
	if (network == 0)
	{
		LOG_START;
		fprintf(log_file,"interface bound to net 0\n");
		LOG_END;
		exit(1);
	}
	n_of_ifaces += 1;

	if (add_route(network,1,2,network,node) == NULL)
	{
		LOG_START;
		fprintf(log_file,
			"cannot add route to iface (out of memory)\n");
		LOG_END;
		exit(1);
	}

	if ((ifc->output.sk = socket(AF_IPX, SOCK_DGRAM, PF_IPX)) < 0)
	{
		LOG_START;
		fprintf(log_file,
			"can't open socket: %s\n", strerror(errno));
		LOG_END;
		exit(1);
	}

	memset(&sipx, 0, sizeof(sipx));
	sipx.sipx_family = AF_IPX;
	sipx.sipx_network = htonl(network);
	ipx_assign_node(sipx.sipx_node, IPX_THIS_NODE);
	sipx.sipx_port = htons(IPX_RIP_PORT);
	sipx.sipx_type = IPX_RIP_PTYPE;

	if (bind(ifc->output.sk, (struct sockaddr *)&sipx, sizeof(sipx)) < 0)
	{
		LOG_START;
		fprintf(log_file, "can't bind socket: %s\n",
			strerror(errno));
		LOG_END;
		exit(1);
	}

	if (ipx_kern_enable_broadcast(ifc->output.sk) != 0)
	{
		LOG_START;
		fprintf(log_file, "cant' enable broadcast\n");
		LOG_END;
		exit(1);
	}
	
	LOG_START;
	ipx_fprint_network(log_file, network);
	fprintf(log_file,":");
	ipx_fprint_node(log_file, node);
	fprintf(log_file,"\n");
	LOG_END;
	return 0;
}

static int
ipx_rip_del_old_rt(IPXNet net, IPXNet rt_net, IPXNode node,
		   int type, void *data)
{
	if ((type & IPX_KRT_ROUTE) != 0)
	{
		/* remove old routes */
		ipx_rip_rt_del(net);
	}
	return 0;
}

static void
int_handler()
{
	signal(SIGINT,int_handler);
	dump_request=1;
}

static void
timer_handler()
{
	signal(SIGALRM,timer_handler);
	timer_request=1;
}

static void
terminate_handler()
{
	signal(SIGTERM,terminate_handler);
	terminate=1;
}

static void
hup_handler()
{
	signal(SIGHUP,hup_handler);
	terminate=2;
}

static void
init()
{
	struct itimerval itval;
	
	LOG_ENTRY;
	LOG_START;
	fprintf(log_file,"Init start\n");
	LOG_END;
	
	ipx_rip_output_func=output_sendto;
	rtable = NULL;
	n_of_ifaces = 0;

	ipx_kern_scan_ifaces(ipx_rip_create_ifc, NULL);
	ipx_kern_scan_rtable(ipx_rip_del_old_rt, NULL);

	itval.it_interval.tv_sec = TIMER_RATE;
	itval.it_value.tv_sec = TIMER_RATE;
	itval.it_interval.tv_usec = 0;
	itval.it_value.tv_usec = 0;
	if (setitimer(ITIMER_REAL, &itval, (struct itimerval *)NULL) < 0)
	{
		LOG_START;
		fprintf(log_file, "FATAL ERROR: can't set itimer: %s\n",
			strerror(errno));
		LOG_END;
		exit(1);
	}
	
	signal(SIGINT,int_handler);
	signal(SIGALRM,timer_handler);
	signal(SIGTERM,terminate_handler);
	signal(SIGHUP,hup_handler);
	
        {
		struct rip_table* rt;
		output_set_destination(IPX_BROADCAST,IPX_RIP_PORT);
		for(rt=rtable;rt!=NULL;rt=rt->next)
		{
			output_broadcast(rt,0);
		}
		output_flushall();
	}
	{
		struct rip_interface* ifc;
		for(ifc=ifaces;ifc<ifaces+n_of_ifaces;ifc++)
		{
			ipx_rip_output_request(&(ifc->output),
					       IPX_RIP_GENERAL_RQ);
		}
		output_flushall();
	}
	LOG_ENTRY;
	LOG_START;
	fprintf(log_file,"Init end\n");
	LOG_END;
}

static void
run(void)
{
	struct sockaddr_ipx sipx;
	int addr_len=sizeof(sipx);
	int size;
	fd_set rset;
	int i, result;
	int max_sk = 0;
	struct rip_interface *ifc = NULL;

	while(1)
	{
		FD_ZERO(&rset);

		for (i = 0; i < n_of_ifaces; i += 1)
		{
			if (ifaces[i].output.sk > max_sk)
			{
				max_sk = ifaces[i].output.sk;
			}
			FD_SET(ifaces[i].output.sk, &rset);
		}		

		LOG_ENTRY;

		result = select(max_sk+1, &rset, NULL, NULL, NULL);

		if (   (result < 0)
		    && (errno != EINTR))
		{
			LOG_START;
			fprintf(log_file, "select error: %s\n",
				strerror(errno));
			LOG_END;
			continue;
		}

		size = -1;

		if (result == 0)
		{
			LOG_START;
			fprintf(log_file, "select timeout expired\n");
			LOG_END;
			continue;
		}

		if (result > 0)
		{
			for (i = 0; i < n_of_ifaces; i += 1)
			{
				if (FD_ISSET(ifaces[i].output.sk, &rset))
				{
					ifc = &(ifaces[i]);
					size=recvfrom(ifaces[i].output.sk,
						      (void*)in_buf,
						      sizeof(in_buf),
						      0,
						      (struct sockaddr*)&sipx,
						      &addr_len);
					/* only one packet at a time */
					break;
				}
			}
		}

		if (dump_request)
		{
			fdump_routes(log_file);
			dump_request=0;
		}
		if (timer_request)
		{
			do_aging();
			timer_request=0;
		}
		if (terminate)
		{
			LOG_START;
			if (terminate==1)
				fprintf(log_file,
					"exiting on signal 15 (SIGTERM)\n");
			else
				fprintf(log_file,
					"exiting on signal 1 (SIGHUP)\n");
			LOG_END;
			return;
		}
		if (size==-1)
		{
			continue;
		}
		DL_START;
		fprintf(log_file,"RIP from: ");
		ipx_fprint_saddr(log_file,&sipx);
		fprintf(log_file,"\n");
		ipx_rip_fdump(log_file,(struct rip_packet *)in_buf,size);
		DL_END;
		handle_rip((struct rip_packet *)in_buf,size,&sipx,ifc);
	}
}

static void
done()
{
	struct rip_table* cur;
	
	LOG_ENTRY;
	LOG_START;
	fprintf(log_file,"Shutdown start\n");
	LOG_END;
	
	output_set_destination(IPX_BROADCAST,IPX_RIP_PORT);
	for(cur=rtable; cur!=NULL; cur=cur->next)
	{
		/* update kernel table (don't delete ifaces)*/
		if (cur->network!=cur->router_network) 
		{
			ipx_rip_rt_del(cur->network);
			LOG_START
			fprintf(log_file,"DELETE ");
			fprint_route(log_file,cur);
			fprintf(log_file," (shutdown)\n");
			LOG_END
		}
		/* send info bcast */
		cur->hops=IPX_RIP_NET_DOWN;
		output_broadcast(cur,1);
		/* update table */
		delete_route(cur);
	}
	output_flushall();

	LOG_ENTRY;
	LOG_START;
	fprintf(log_file,"Shutdown end\n");
	LOG_END;
}

static void
help()
{
	fprintf(stderr,
		"IPX routing daemon v0.91, (c) Ales Dryak, 1995\n"
		"Usage: ipxripd [-d] [<path to log file>]\n");
}

static int
parse_cmdline(int argc,char** argv)
{
	char* log_name;
	argc--,argv++;
	for(; *argv && **argv=='-'; argc--,argv++)
	{
		if (*(*argv+1)=='d')
			debug_option=1;
		else
		{
			fprintf(stderr,
				"Unknown command line option %s\n",*argv);
			help();
			exit(1);
		}
	}
	if (argc>1)
	{
		fprintf(stderr,"Too many parameters");
		help();
		exit(1);
	}
	if (argc==0 && debug_option)
	{
		log_file=stdout;
		return 0;
	}
	log_name=DEFAULT_LOGNAME;
	if (argc==1) log_name=*argv;
	if ((log_file=fopen(log_name,"a"))==NULL)
	{
		perror(log_name);
		exit(1);
	}
	return 1;
}

static void
daemonize()
{
	int fd,c;
	
	if ((c = fork()) > 0) exit(0);
  	if (c < 0)
  	{
		fprintf(stderr, "ipxripd: can't fork: %s\n",strerror(errno));
		exit(1);
	}

	close(0);
	close(1);
	close(2);
	if ((fd = open("/dev/tty", O_RDWR)) >= 0) 
	{
		ioctl(fd, TIOCNOTTY, NULL);
		close(fd);
	}
}

int
main(int argc,char** argv)
{
	if (parse_cmdline(argc,argv)) daemonize();
	init();
	run();
	done();
	return 0;
}
